﻿using UnityEngine;
using System.Collections;

// 参考資料
// http://docs.unity3d.com/ja/current/Manual/InstantiatingPrefabs.html
using System.Xml.Serialization;

public class Spectrum : MonoBehaviour
{
	public GameObject prefab;
	public int numberOfObjects = 20;
	public float radius = 5f;
	public GameObject[] cubes;
	private AudioListener AL;
	private AudioSource AS;
	public Material mat;
	public GameObject cameraPivot;
	public GameObject textOf3D;
	public GameObject yukari;

	public float maxTime;
	public float currentTime;

	float[] samples = new float[1024];

	[Header ("User Config")]
	public float r = 0f;
	public float g = 0f;
	public float b = 0f;
	public float volume = 0f;
	private Color color;

	void Start ()
	{
		AS = GetComponent<AudioSource> ();
		AL = GetComponent<AudioListener> ();

		maxTime = AS.clip.length;

		// color by materials
		r = PlayerPrefs.GetFloat ("R");
		g = PlayerPrefs.GetFloat ("G");
		b = PlayerPrefs.GetFloat ("B");
		color = new Color (r, g, b);

		volume = PlayerPrefs.GetFloat ("volume");


		for (int i = 0; i < numberOfObjects; i++) {
			float angle = i * Mathf.PI * 2 / numberOfObjects;
			Vector3 pos = new Vector3 (Mathf.Cos (angle), 0, Mathf.Sin (angle)) * radius;
			Instantiate (prefab, pos, Quaternion.identity);
		}

		cubes = GameObject.FindGameObjectsWithTag ("cubes");

		AS.time = currentTime;
	}

	void Update ()
	{
		currentTime = AS.time;
		// GetOutputDataは音量
		//AudioListener.GetOutputData (samples, 1);
		AudioListener.GetSpectrumData (samples, 0, FFTWindow.BlackmanHarris);

		for (int i = 0; i < numberOfObjects; i++) {
			Vector3 previousScale = cubes [i].transform.localScale;
			previousScale.y = Mathf.Lerp (previousScale.y, samples [i] * 40, Time.deltaTime * 30);
			cubes [i].transform.localScale = previousScale;
			cubes [i].GetComponent<MeshRenderer> ().material.SetColor ("_TintColor", color);
			cubes [i].GetComponent<MeshRenderer> ().material.color = color;
		}

		Vector3 preScale = yukari.transform.localScale;
		//preScale.x = Mathf.Lerp (preScale.x, 1 + samples [0] * 10, Time.deltaTime * 30);
		preScale.y = Mathf.Lerp (preScale.y, 1 + samples [0] * 1, Time.deltaTime * 30);
		//preScale.z = Mathf.Lerp (preScale.z, 1 + samples [0] * 10, Time.deltaTime * 30);
		yukari.transform.localScale = preScale;


		//cameraPivot.transform.Rotate (0, 0.25f, 0);
		//textOf3D.transform.Rotate (0, -0.25f, 0);
	}

	void OnGUI ()
	{
		r = GUI.HorizontalSlider (new Rect (20, 10, Screen.width - 40, 20), r, 0f, 1f);
		g = GUI.HorizontalSlider (new Rect (20, 30, Screen.width - 40, 20), g, 0f, 1f);
		b = GUI.HorizontalSlider (new Rect (20, 50, Screen.width - 40, 20), b, 0f, 1f);

		volume = GUI.HorizontalSlider (new Rect (20, 70, Screen.width - 40, 20), volume, 0f, 1f);
		currentTime = GUI.HorizontalSlider (new Rect (20, Screen.height - 30, Screen.width - 40, 20), currentTime, 0f, maxTime);

		/*
		for (int i = 0; i < numberOfObjects; i++) {
			cubes [i].gameObject.GetComponent<Renderer>().material = mat;
		}
		*/

		PlayerPrefs.SetFloat ("R", r);
		PlayerPrefs.SetFloat ("G", g);
		PlayerPrefs.SetFloat ("B", b);
		PlayerPrefs.SetFloat ("volume", volume);

		//mat.color = new Color (r, g, b);
		color = new Color (r, g, b);

		AS.volume = volume;
	}
}
